package dev.kske.undoredo.core;

import java.util.Objects;

/**
 * @author Kai S. K. Engelbart
 * @since 0.0.1
 */
class IntChange implements Change {

	private final IntWrapper	wrapper;
	private final int			value;

	IntChange(IntWrapper wrapper, int value) {
		this.wrapper	= wrapper;
		this.value		= value;
	}

	@Override
	public void apply() {
		wrapper.value += value;
	}

	@Override
	public Change invert() {
		return new IntChange(wrapper, -value);
	}

	@Override
	public boolean isIdentity() {
		return value == 0;
	}

	@Override
	public String toString() {
		return String.valueOf(value);
	}

	@Override
	public int hashCode() {
		return Objects.hash(value, wrapper);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof IntChange))
			return false;
		IntChange other = (IntChange) obj;
		return value == other.value && Objects.equals(wrapper, other.wrapper);
	}
}
